#ifndef __Aip4cComponents_Settings_h__
#define __Aip4cComponents_Settings_h__

#include <Aip4cComponents/Aip4cComponents.h>
#include <Aip4cComponents/Exception.h>
#include <Aip4cComponents/PluginInfo.h>

#include <librfc/XMLDoc.hpp>

class QWidget;

namespace Aip4cComponents
{
	class Aip4cComponents_Export Settings
	{
		public:
			class Product
			{
				private:
					int _nProduct;
					std::string _strName;
					std::string _strPlugin;

				public:
					Product( int nProduct = -1, const std::string& strName = std::string(), const std::string& strPlugin = std::string() );
					Product( const Product& product );

					const Product& operator=( const Product& product );

					int product() const;
					const std::string& name() const;
					const std::string& plugin() const;
			};

			class ProductMap : public std::map<int, Product>
			{
			};

			class ProductPtrMap : public std::map<int, Product*>
			{
			};

		private:
			bool _bAdmin;
#if defined (LINUX)
			std::string _strHKCU;
			std::string _strHKLM;
			Rfc::XMLDoc _docHKCU;
			Rfc::XMLDoc _docHKLM;
			Rfc::XMLNode* _keyHKCU;
			Rfc::XMLNode* _keyHKLM;
			Rfc::XMLNode* _keyRoot4cCU;
			Rfc::XMLNode* _keyRoot4cLM;
			Rfc::XMLNode* _keyRootAip4cCU;
			Rfc::XMLNode* _keyRootAip4cLM;
#elif defined (_WINDOWS)
			HKEY _keyRoot4cCU;
			HKEY _keyRoot4cLM;
			HKEY _keyRootAip4cCU;
			HKEY _keyRootAip4cLM;
#endif /* LINUX */
			ProductMap _productS;
			ProductPtrMap _tabS;

		public:
			Settings();
			virtual ~Settings();

			void load( bool bAdmin = false ) throw( Aip4cComponents::Exception );
			const ProductMap& products() const;
			ProductMap& products();
			const ProductPtrMap& tabs() const;
			ProductPtrMap& tabs();
			bool save();

			void setPlugin( const std::string& strFile, const PluginInfo& info ) throw( Aip4cComponents::Exception );

			static int doSettings( Settings& settings, QWidget* pparent = NULL );
			static void registerPlugin( Settings& settings, const std::string& strFile ) throw( Aip4cComponents::Exception );
			static void unregisterPlugin( Settings& settings, const std::string& strFile ) throw( Aip4cComponents::Exception );
	};
}

#endif /* !__Aip4cComponents_Settings_h__ */
